/* filename:    lppm.g
** description: The program maximizes the posterior density of the 
**              DSGE model conditional on phi
** created:     02/06/02
*/

new;
library pgraph, user;
cls;

format /mat /on /mb1 /ros 16,8;

sscond = 0;

/* Global defaults
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g

/******************************************************** 
** Import data
** series (nobs,3)
*/
#include c:\projects\active\learnpol\gauss2\loaddata.g

nobs  = rows(series_yT);  /* number of observations */
YY    = series_yT;   
SS    = series_sT;

/******************************************************** 
** Import the information for the prior density of theta
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prt.out";
load path = ^priopath prior_t[nparat,3] = ^priorfile;

pmean_t  = prior_t[.,1];
pstdd_t  = prior_t[.,2];
pshape_t = prior_t[.,3];
pshape_t = pshape_t.*parat_maskinv[.,mspec];

/******************************************************** 
** Import the information for the prior density of phi
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prp.out";
load path = ^priopath prior_p[nparap,3] = ^priorfile;

pmean_p  = prior_p[.,1];
pstdd_p  = prior_p[.,2];
pshape_p = prior_p[.,3];
pshape_p = pshape_p.*parap_maskinv[.,mspec];

/******************************************************** 
** Starting Values for Maximization
*/

gam   = 0.4;    /* quarterly productivity growth */
pila  = 3;      /* annualized inflation (low) */
pida  = 4;      /* annualized inflation differential (high-low) */
rsta  = 2;      /* annualized real interest rate */

tau   = 0.6;    /* substitution elasticity */
kap   = 0.5;    /* PC slope */
psi   = 1.5;    /* Policy rule: inflation */

rhog  = 0.9;    /* Persistence: g-shock */
rhoz  = 0.3;    /* Persistence: technology growth */
rhor  = 0.7;    /* Interest rate smoothing */

sigg   = 1.2;
sigz   = 0.3;
sigrla = 0.6;
sigrha = 1.6;

para_t  = gam  | pila | pida  | rsta |
          tau  | kap  | psi   |
          rhog | rhoz | rhor  |
          sigg | sigz | sigrla | sigrha ;

para_t  = para_t.*parat_maskinv[.,mspec] + parat_fix[.,mspec].*parat_mask[.,mspec];

para_p  = 0.8 | 0.8;

para_p  = para_p.*parap_maskinv[.,mspec] + parap_fix[.,mspec].*parap_mask[.,mspec];

parat_names = { GAM PILA PIDA RSTA TAU KAP PSI RoG RoZ RoR SigG SigZ SRLA SRHA };
save path = ^priopath parat_names;

parap_names = { PHI1 PHI2 };
save path = ^priopath parap_names;

/* Load transformation scheme for parameters
*/
transfile = "transt.out";
load path=^priopath _trspect[nparat,4] = ^transfile;
_trspect[.,1] = _trspect[.,1].*parat_maskinv[.,mspec];

transfile = "transp.out";
load path=^priopath _trspecp[nparap,4] = ^transfile;
_trspecp[.,1] = _trspecp[.,1].*parap_maskinv[.,mspec];


/* Stack Parameters
*/
para  = para_t | para_p;
para_names = parat_names ~ parap_names;

pmean  = pmean_t | pmean_p;
pstdd  = pstdd_t | pstdd_p;
pshape = pshape_t | pshape_p;

_trspec = _trspect | _trspecp;

/******************************************************** 
**      Calculate posterior density at starting values
**
*/
cls;
"Prior Density at Starting Value:" priodens(para,pmean,pstdd,pshape);
lnpy = -objfcn(invtrans(para));
"Posterior at Starting Value:    " lnpy;
"Press key to continue";
w = keyw;
cls;

/********************************************************
**       Maximize the posterior density
**
*/
cc = 0;
x0 = invtrans(para) + cc* ( para_maskinv[.,mspec].*rndn(npara,1) );

H0= eye(npara)*1E-4;
nit = 1000;
crit= 1E-7;

/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/********************************************************
** Save parameter estimates
*/
paraest = trans(xh);
paraest = real( paraest.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec] );

oparaest         = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
create fhparaest = ^oparaest with PARAEST, 1, 8;
wr               = writer(fhparaest,paraest);
closeall fhparaest;   


/********************************************************
**       Estimation Output
**
*/
if sscond == 1; 
   /* Condition on states */
   {lnpY,retcode,obserror,obsvar} = evallpss(paraest[1:nparat],paraest[nparat+1:npara],mspec,YY,SS); 
else;
   /* Integrate over states */
   {lnpY,retcode,obserror,obsvar} = evallp(paraest[1:nparat],paraest[nparat+1:npara],mspec,YY); 
endif;

lnprio  = priodens( paraest, pmean, pstdd, pshape);

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

cls;
"Model              " mspec;
"Dataset            " datasel;
"Subsample          " SubT;
"===================================";
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
" ";
"Parameter   | Estimate |  Start-V  | Prior Mean | Prior Stdd | Gradient";
outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
let mask[1,6] = 0 1 1 1 1 1;
let fmt[6,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,(0~1~1~1~1~1),fmt);

if lnpy == -1E6 ;
   end;
endif;

/********************************************************
**       Compute Residuals
**
*/

/* Moments */

resid1 = obserror;
resid1 = resid1[2:rows(resid1),.];

resid1_m1 = meanc(resid1)';
resid1_m2 = meanc( (resid1 - resid1_m1)^2 )';
resid1_m3 = meanc( (resid1 - resid1_m1)^3 )';
resid1_m4 = meanc( (resid1 - resid1_m1)^4 )';

resid2 = obserror./sqrt(obsvar);
resid2 = resid2[2:rows(resid2),.];
resid2_m1 = meanc(resid1)';
resid2_m2 = meanc( (resid2 - resid2_m1)^2 )';
resid2_m3 = meanc( (resid2 - resid2_m1)^3 )';
resid2_m4 = meanc( (resid2 - resid2_m1)^4 )';

ti = ti[2:rows(ti)];

nresid = rows(resid1);

/* Autocorrelation */

resid1_ar = zeros(1,3);
resid2_ar = zeros(1,3);

ind = 1;
do until ind > 3;
   resid1_ar[1,ind]=inv(resid1[1:nresid-1,ind]'*resid1[1:nresid-1,ind])
                    *resid1[1:nresid-1,ind]'*resid1[2:nresid,ind];
   resid2_ar[1,ind]=inv(resid2[1:nresid-1,ind]'*resid2[1:nresid-1,ind])
                    *resid2[1:nresid-1,ind]'*resid2[2:nresid,ind];
   ind = ind+1;
endo;


"Raw Residuals";
"-------------";
"Mean:" resid1_m1;
"Stdd:" sqrt(resid1_m2);
"Skew:" resid1_m3./resid1_m2^(3/2);
"Kurt:" resid1_m4./(3*resid1_m2^2);
"Corr:" resid1_ar;
"";
"Standardized Residuals";
"----------------------";
"Mean:" resid2_m1;
"Stdd:" sqrt(resid2_m2);
"Skew:" resid2_m3./resid2_m2^(3/2);
"Kurt:" resid2_m4./(3*resid2_m2^2);
"Corr:" resid2_ar;
"";

titlestr = "Output Growth"  $|
           "Inflation" $|
           "Interest Rate";

ystr     = "Raw Residual " $|
           "Stdd Residual"; 

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
          
begwind;
window(2,3,0);


ser_ind = 1;
do until ser_ind > 3;

   ylabel(ystr[1]);
   title(titlestr[ser_ind]);
   xy(ti,resid1[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

ser_ind = 1;
do until ser_ind > 3;

   ylabel(ystr[2]);
   title(titlestr[ser_ind]);
   xy(ti,resid2[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

endwind;
end;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = objfcn(para);
local lnpY, lnprio, obserror, obsvar, modelpara, retcode;

/* likelihood 
*/
modelpara = trans(para);
modelpara = modelpara.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];

if sscond == 1; 
   /* Condition on states */
   {lnpY,retcode,obserror,obsvar} = evallpss(modelpara[1:nparat],modelpara[nparat+1:npara],mspec,YY,SS); 
else;
   /* Integrate over states */
   {lnpY,retcode,obserror,obsvar} = evallp(modelpara[1:nparat],modelpara[nparat+1:npara],mspec,YY); 
endif;

if ismiss(lnpY);
   lnpY = -1E8;
endif;

/* Evaluate the Prior distribution
*/
lnprio = priodens(modelpara, pmean, pstdd, pshape);

retp(real(-lnpY-lnprio));  /* We minize the inverse of the likelihood fcn */
endp;

#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lplh.src;
#include c:\projects\active\learnpol\gauss2\lpdraw.src;
